// @flow

export type PersistState = {
  version: number,
  rehydrated: boolean,
}

export type PersistedState = {
  _persist: PersistState,
} | void

export type PersistConfig = {
  version?: number,
  storage: Object,
  key: string,
  keyPrefix?: string,
  debug?: boolean,
  blacklist?: Array<string>,
  whitelist?: Array<string>,
  transforms?: Array<Transform>,
  throttle?: number,
  migrate?: (ps: PersistedState, n: number) => Promise<PersistedState>,
  getStoredState?: (pc: PersistConfig) => Promise<PersistedState>,
}

export type PersistorOptions = {
  enhancer?: Function,
}

export type Storage = {
  getItem: (key: string, cb: ?(s: string) => any) => any,
  setItem: (key: string, value: string, cb: ?() => any) => any,
  removeItem: (key: string, cb: ?() => any) => any,
}

type ManifestKey = number | string
export type MigrationManifest = {
  [key: ManifestKey]: (ps: PersistedState) => PersistedState,
}

export type Transform = {
  in: (s: Object, key: string) => Object,
  out: (s: Object, key: string) => Object,
  config?: PersistConfig,
}

export type RehydrateErrorType = any

export type RehydrateAction = {
  type: 'redux-persist/REHYDRATE',
  key: string,
  payload: ?Object,
  err: ?RehydrateErrorType,
}

export type Persistoid = {
  update: (state: Object) => void,
}

type RegisterAction = {
  type: 'redux-persist/REGISTER',
  key: string,
}

type PersistorAction = RehydrateAction | RegisterAction

type PersistorState = {
  registry: Array<string>,
  bootstrapped: boolean,
}

type PersistorSubscribeCallback = () => void

export type Persistor = {
  purge: () => void,
  +dispatch: (pa: PersistorAction) => PersistorAction,
  +getState: () => PersistorState,
  +subscribe: (cb: PersistorSubscribeCallback) => () => void,
}
